package xxx;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.HashMap;

/**
 * Diese Klasse dient zur Key-Generation fr JDBC . &lt;br&gt;
 * Dieser KeyGenerator holt sich beim ersten Zugriff ber
 * &lt;pre&gt;
 * SELECT MAX( &amp;lt;field&amp;gt; ) FROM &amp;lt;table&amp;gt;
 * &lt;/pre&gt;
 * den maximalen Key von der DB-Tabelle und zhlt von da an nur noch im
 * Speicher weiter.
 * &lt;br&gt;&lt;br&gt;
 * &lt;b&gt;
 * !!! Achtung Achtung Achtung !!!
 * &lt;br&gt;
 * Dieser einfache KeyGenerator funktioniert nur, wenn
 * ausschliesslich aus der VM-Instanz auf die Datenbank schreibend
 * zugegriffen wird, in welcher dieser Code luft.
 * &lt;/b&gt;
 *
 * @author Heiner Kcker
 * @since 13.11.2004
 */
public class KeyGenerator
{
   /**
    * Speicher fr vorgemerkte Keys,
    * damit nicht bei jedem Key auf die
    * Datenbank gegangen werden muss.
    * Ebene Datenbank.
    * @see {@link #keyStep}
    */
   private static HashMap dbMemoryKeyMap = new HashMap();

   /**
    * Methode zur Key-Generation fr JDBC . &lt;br&gt;
    *
    * @param dbName         Name der Datenbank
    * @param tableName      Name der Datenbank-Tabelle
    * @param genKeyFldName  Name des Primery-Key Feldes
    * @return der nchste freie Key (Long, INTEGER, BIGINT, DECIMAL)
    */
   synchronized public static long getNextKey(
      String pDbName,
      String pTableName,
      String pFldName)
   {
      //Abfrage der Tabellen-Map auf DB-Ebene
      HashMap tblKeyHashMap = (HashMap) dbMemoryKeyMap.get( pDbName );

      if (tblKeyHashMap == null)
      {
         // neue Tabellen-Map fr eine DB anlegen
         tblKeyHashMap = new HashMap();
         dbMemoryKeyMap.put( pDbName, tblKeyHashMap );
      }

      //Abfrage auf Tabellen-Ebene
      KeyEntry keyEntry = (KeyEntry) tblKeyHashMap.get( pTableName );
      if (keyEntry == null)
      {
         keyEntry = new KeyEntry();
         tblKeyHashMap.put( pTableName , keyEntry );

         Long lastKeyObj = getMaxFromDbTable(pDbName, pTableName, pFldName);
         if (lastKeyObj != null)
         {
            //es gibt schon Datenstze
            keyEntry.lastKey = lastKeyObj.longValue();
         }
      }
      long nextKey = ++keyEntry.lastKey;
      return nextKey;
   }

   /**
    * Diese Klasse trgt die im Speicher gehaltenen Keys.
    *
    * @author Heiner Kcker
    */
   private static class KeyEntry
   {
      /**
       * letzter vergebener Key . &lt;br&gt;
       * Zhlung beginnt mit 1.&lt;br&gt;
       * Null steht fr undefiniert.&lt;br&gt;
       */
      long lastKey = 0;
   }

   /**
    * Abfrage des maximalen Keys von der Datenbank.
    * Wenn der Key unbekannt ist, wird null zurckgegeben.
    *
    * @param dbName         Name der Datenbank
    * @param tableName      Name der Datenbank-Tabelle
    * @param genKeyFldName  Name des Primery-Key Feldes
    */
   private static synchronized Long getMaxFromDbTable(
      final String pDbName,
      final String pTableName,
      final String pFldName)
   {
      java.sql.Connection con = null;

      try
      {
         // hier eigene Vorgehensweise zum Holen der Datenbank-Connection einbauen
         con = DatabaseConnectionFactory.getConnection( pDbName );

         String selectStr = "SELECT MAX(" + pFldName + ") FROM " + pTableName;

         PreparedStatement myPreparedStatement = con.prepareStatement(selectStr);
         ResultSet rsltSet = myPreparedStatement.executeQuery();

         if (rsltSet != null &amp;&amp; rsltSet.next())
         {
            long maxKey = rsltSet.getLong(1);
            if (rsltSet.wasNull())
            {
               return null;
            }
            return new Long(maxKey);
         }
         else
         {
            //erstes Anlegen des Key fr diese Tabelle
            return null;
         }
      }
      catch (Exception e)
      {
         System.err.println( e.getMessage() );
         e.printStackTrace();
         return null;
      }
      finally
      {
         if (con != null)
         {
            try
            {
               con.close();
            }
            catch (Exception e)
            {
               System.err.println( e.getMessage() );
               e.printStackTrace();
               return null;
            }
         }
      }

   }

}
